#!/bin/bash

# --------------------------------------------------------------------------- #
function showUsage {
    echo "Использование: $(basename $0) coredump [binfile]"
    echo "    coredump - путь к дампу"
    echo "     binfile - (опционально) путь к исполняемому файлу платформы, упавшему в дамп"
    echo "               используется для случаев, когда не удалось загрузить дамп автоматически."
}

# --------------------------------------------------------------------------- #
function exit_cleanup {
#    rm -f "$tmpCommandsFile" "$tmpLibsListFile"
    true
}

coredumpPath="$1"
binfilePath="$2"
if [ -z "$coredumpPath" -o "$1" = "--help" ] ; then
    showUsage
    exit 1
fi

trap exit_cleanup EXIT

tmpCommandsFile=$(mktemp)
tmpLibsListFile=$(mktemp)

cat <<EOT > "$tmpCommandsFile"
info shared
quit
EOT

if [ -z "$binfilePath" ]; then
    gdb -x "$tmpCommandsFile" -c "$coredumpPath"  2>/dev/null > "$tmpLibsListFile"
else
    gdb -x "$tmpCommandsFile" "$binfilePath" "$coredumpPath"  2>/dev/null > "$tmpLibsListFile"
fi

# Считаем, что в дампе должен быть путь к core83.so. По нему определяем путь к
# бинарным файлам платформы. Далее все файлы, начинающиеся с этого пути, исключаем
# из списка библиотек
core83path="$(cat "$tmpLibsListFile" | grep core83.so | awk '{print $NF;}')"

if [ -z "$core83path" ]; then
    if grep -q "No shared libraries loaded at this time" "$tmpLibsListFile"; then
        echo "Ошибка: Нe удалось загрузить дамп."
        echo "        Выполните повторный запуск скрипта, передав в качестве второго параметра"
        echo "        путь к исполняемому файлу платформы, сгенерировавшему дамп"
        exit 1
    fi
fi
v8BinPath=$(dirname "$core83path")

if [ -z "$v8BinPath" ]; then
    echo "В дампе не удалось найти core83.so, дальнейший анализ невозможен."
    exit 1
fi

echo "Нашли путь к бинарным файлам платформы: $v8BinPath"

# Определяем список сторонних библиотек в дампе
libColumn=$(cat "$tmpLibsListFile" | grep "core83.so" | perl -e "print 1+index(<STDIN>, \"$v8BinPath\")")

# формируем скрипт для формирования архива с библиотеками
getLibsScriptPath="${coredumpPath}_getlibs.sh"
archivePath="libs-$(basename ${coredumpPath}).tar.gz"
echo "echo Generating \"$archivePath\"" > "$getLibsScriptPath"
echo "tar hcfz \"$archivePath\" \\" >> "$getLibsScriptPath"
cat "$tmpLibsListFile" | grep -v "$v8BinPath" | cut -b"$libColumn"- | grep "^/" | while read lib; do
    echo "   $lib \\" >> "$getLibsScriptPath"
done
echo >> "$getLibsScriptPath"

echo
echo "Сгенерирован скрипт сбора библиотек: $getLibsScriptPath"
echo "После его запуска на целевой системе будет создан архив $archivePath"

